// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xparameters.h"
#include "xuartlite_l.h"

#define JESD0_BASEADDR      XPAR_AXI_JESD204B_RX4_0_BASEADDR
#define JESD1_BASEADDR      XPAR_AXI_JESD204B_RX4_1_BASEADDR
#define DMA0_0_BASEADDR     XPAR_AXI_DMA_0_0_BASEADDR
#define DMA0_1_BASEADDR     XPAR_AXI_DMA_0_1_BASEADDR
#define DMA1_0_BASEADDR     XPAR_AXI_DMA_1_0_BASEADDR
#define DMA1_1_BASEADDR     XPAR_AXI_DMA_1_1_BASEADDR
#define DDR_BASEADDR        XPAR_DDR3_SDRAM_S_AXI_BASEADDR
#define UART_BASEADDR       XPAR_RS232_UART_1_BASEADDR
#define SPI0_BASEADDR       XPAR_AXI_SPI_0_BASEADDR
#define SPI1_BASEADDR       XPAR_AXI_SPI_1_BASEADDR
#define GPIOLCD_BASEADDR    XPAR_AXI_GPIO_0_BASEADDR
#define CF_ADC0_0_BASEADDR  XPAR_AXI_AD9250_0_0_BASEADDR
#define CF_ADC0_1_BASEADDR  XPAR_AXI_AD9250_0_1_BASEADDR
#define CF_ADC1_0_BASEADDR  XPAR_AXI_AD9250_1_0_BASEADDR
#define CF_ADC1_1_BASEADDR  XPAR_AXI_AD9250_1_1_BASEADDR

#define FMC176_CPLD      0x00
#define FMC176_AD9517    0x84
#define FMC176_AD9250_0  0x80
#define FMC176_AD9250_1  0x81
#define FMC176_AD9129_0  0x82
#define FMC176_AD9129_1  0x83

// ***************************************************************************
// ***************************************************************************

extern char inbyte(void);
void xil_printf(const char *ctrl1, ...);

void delay_ms(u32 ms_count) {
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

u32 user_exit(void) {
  while (!XUartLite_IsReceiveEmpty(UART_BASEADDR)) {
    if (XUartLite_RecvByte(UART_BASEADDR) == 'q') {
      return(1);
    }
  }
  return(0);
}

void lcd_write_1(u32 data) {
  u32 count;
  Xil_Out32(GPIOLCD_BASEADDR, (0x00 | (data&0x3f)));
  Xil_Out32(GPIOLCD_BASEADDR, (0x00 | (data&0x3f)));
  for (count = 0; count < 13; count++) {
    asm("nop");
  }
  Xil_Out32(GPIOLCD_BASEADDR, (0x40 | (data&0x3f)));
  Xil_Out32(GPIOLCD_BASEADDR, (0x40 | (data&0x3f)));
  for (count = 0; count < 26; count++) {
    asm("nop");
  }
  Xil_Out32(GPIOLCD_BASEADDR, (0x00 | (data&0x3f)));
  Xil_Out32(GPIOLCD_BASEADDR, (0x00 | (data&0x3f)));
  for (count = 0; count < 13; count++) {
    asm("nop");
  }
}

void lcd_write(u32 cntrl, u32 data) {
  u32 count;
  lcd_write_1((((cntrl<<4)&0x30) | ((data>>4)&0xf)));
  for (count = 0; count < 100; count++) {
    asm("nop");
  }
  lcd_write_1((((cntrl<<4)&0x30) | ((data>>0)&0xf)));
  for (count = 0; count < 4000; count++) {
    asm("nop");
  }
}

void lcd_display() {

  Xil_Out32(GPIOLCD_BASEADDR, 0x00);
  Xil_Out32((GPIOLCD_BASEADDR + 0x04), 0x00); // outputs

  delay_ms(16);
  lcd_write_1(0x3);
  delay_ms(5);
  lcd_write_1(0x3);
  delay_ms(1);
  lcd_write_1(0x3);
  delay_ms(1);
  lcd_write_1(0x2);
  delay_ms(1);

  lcd_write(0x0, 0x28);
  lcd_write(0x0, 0x06);
  lcd_write(0x0, 0x0c);
  lcd_write(0x0, 0x01);
  delay_ms(2);

  lcd_write(0x0, 0x02);
  lcd_write(0x2, 0x41);
  lcd_write(0x2, 0x6e);
  lcd_write(0x2, 0x61);
  lcd_write(0x2, 0x6c);
  lcd_write(0x2, 0x6f);
  lcd_write(0x2, 0x67);
  lcd_write(0x2, 0x20);
  lcd_write(0x2, 0x44);
  lcd_write(0x2, 0x65);
  lcd_write(0x2, 0x76);
  lcd_write(0x2, 0x69);
  lcd_write(0x2, 0x63);
  lcd_write(0x2, 0x65);
  lcd_write(0x2, 0x73);
  lcd_write(0x2, 0x20);

  lcd_write(0x0, 0xa8);
  lcd_write(0x2, 0x32);
  lcd_write(0x2, 0x41);
  lcd_write(0x2, 0x44);
  lcd_write(0x2, 0x39);
  lcd_write(0x2, 0x32);
  lcd_write(0x2, 0x35);
  lcd_write(0x2, 0x30);
  lcd_write(0x2, 0x2d);
  lcd_write(0x2, 0x32);
  lcd_write(0x2, 0x41);
  lcd_write(0x2, 0x44);
  lcd_write(0x2, 0x39);
  lcd_write(0x2, 0x32);
  lcd_write(0x2, 0x35);
  lcd_write(0x2, 0x30);
  lcd_write(0x2, 0x20);

  xil_printf("Analog Devices, 2AD9250-2AD9250\n\r");
}

void fmc176_spi_write(u32 bsel, u32 sel, u32 addr, u32 data) {
  u32 spi_baseaddr;
  spi_baseaddr = (bsel == 1) ? SPI1_BASEADDR : SPI0_BASEADDR;
  Xil_Out32((spi_baseaddr + 0x70), 0x001); // chip select
  Xil_Out32((spi_baseaddr + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((spi_baseaddr + 0x70), 0x000); // chip select
  Xil_Out32((spi_baseaddr + 0x68), (sel & 0xff)); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(spi_baseaddr + 0x6c);
  if ((sel == FMC176_AD9517) || (sel == FMC176_AD9250_0) || (sel == FMC176_AD9250_1)) {
    Xil_Out32((spi_baseaddr + 0x68), ((addr>>8) & 0xff)); // mosi data
    Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
    while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
    Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
    Xil_In32(spi_baseaddr + 0x6c);
  }
  Xil_Out32((spi_baseaddr + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(spi_baseaddr + 0x6c);
  Xil_Out32((spi_baseaddr + 0x68), data); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(spi_baseaddr + 0x6c);
  Xil_Out32((spi_baseaddr + 0x70), 0x001); // chip select
  Xil_Out32((spi_baseaddr + 0x60), 0x180); // control register
  delay_ms(1);
}

u32 fmc176_spi_read(u32 bsel, u32 sel, u32 addr, u32 display) {
  u32 rdata;
  u32 spi_baseaddr;
  rdata = 0x80;
  spi_baseaddr = (bsel == 1) ? SPI1_BASEADDR : SPI0_BASEADDR;
  Xil_Out32((spi_baseaddr + 0x70), 0x001); // chip select
  Xil_Out32((spi_baseaddr + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((spi_baseaddr + 0x70), 0x000); // chip select
  Xil_Out32((spi_baseaddr + 0x68), (sel & 0xff)); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(spi_baseaddr + 0x6c);
  if ((sel == FMC176_AD9517) || (sel == FMC176_AD9250_0) || (sel == FMC176_AD9250_1)) {
    Xil_Out32((spi_baseaddr + 0x68), (rdata | ((addr>>8) & 0xff))); // mosi data
    Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
    while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
    Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
    Xil_In32(spi_baseaddr + 0x6c);
    rdata = 0x00;
  }
  Xil_Out32((spi_baseaddr + 0x68), (rdata | (addr & 0xff))); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(spi_baseaddr + 0x6c);
  Xil_Out32((spi_baseaddr + 0x68), 0x000); // mosi data
  Xil_Out32((spi_baseaddr + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((spi_baseaddr + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((spi_baseaddr + 0x60), 0x186); // control register (rst fifos)
  rdata = Xil_In32(spi_baseaddr + 0x6c);
  Xil_Out32((spi_baseaddr + 0x70), 0x001); // chip select
  Xil_Out32((spi_baseaddr + 0x60), 0x180); // control register
  delay_ms(1);
  if (display == 1) {
    xil_printf("Board[%d]-", bsel);
    if (sel == FMC176_CPLD)      xil_printf("FMC176_CPLD");
    if (sel == FMC176_AD9517)    xil_printf("FMC176_AD9517");
    if (sel == FMC176_AD9250_0)  xil_printf("FMC176_AD9250_0");
    if (sel == FMC176_AD9250_1)  xil_printf("FMC176_AD9250_1");
    if (sel == FMC176_AD9129_0)  xil_printf("FMC176_AD9129_0");
    if (sel == FMC176_AD9129_1)  xil_printf("FMC176_AD9129_1");
    xil_printf("[%04x]: %02x\n\r", addr, rdata);
  }
  return(rdata);
}

void fmc176_spi_verify(u32 bsel, u32 sel, u32 addr, u32 data) {
  u32 rdata;
  rdata = fmc176_spi_read(bsel, sel, addr, 0);
  if (rdata != data) {
    xil_printf("Board[%d]-", bsel);
    if (sel == FMC176_CPLD)      xil_printf("FMC176_CPLD");
    if (sel == FMC176_AD9517)    xil_printf("FMC176_AD9517");
    if (sel == FMC176_AD9250_0)  xil_printf("FMC176_AD9250_0");
    if (sel == FMC176_AD9250_1)  xil_printf("FMC176_AD9250_1");
    if (sel == FMC176_AD9129_0)  xil_printf("FMC176_AD9129_0");
    if (sel == FMC176_AD9129_1)  xil_printf("FMC176_AD9129_1");
    xil_printf("[%04x]: rcv(%02x), exp(%02x) !!!\n\r", addr, rdata, data);
  }
  return;
}

u32 ad9250_sel(u32 sel) {
  if (sel == 0) return(FMC176_AD9250_0);
  if (sel == 1) return(FMC176_AD9250_1);
  return(-1);
}

u32 ad9517_setup(u32 bsel) {

  // reference clock is 30.72MHz. vco is at 2.4576GHz (ref1/not-lvds)
  // r divider = 1, n divider 80 (a = 0, b = 5), prescaler- 16/17 DM Mode
  // adc1 (out0) and adc0 (out1) are at 245.76MHz
  // dac1 (out3) and dac0 (out2) are at 2.4576GHz (vco clock)
  // fpga clock (out4) is at 245.76MHz
  // gtx clock (out5) is at 245.76MHz
  // out clock (out6) is at 245.76MHz

  fmc176_spi_write(bsel, FMC176_AD9517, 0x0010, 0x7c);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0014, 0x05);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0016, 0x05);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x001c, 0x02);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x00f1, 0x08);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x00f5, 0x08);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0140, 0x42);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0141, 0x42);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0142, 0x42);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0143, 0x43);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0190, 0x21);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0191, 0x00);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0197, 0x80);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0198, 0x02);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0199, 0x21);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x019c, 0x20);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x019e, 0x21);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x01a0, 0x11);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x01a1, 0x20);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x01e0, 0x00);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x01e1, 0x02);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0232, 0x01);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0018, 0x06);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0232, 0x01);
  delay_ms(1);

  fmc176_spi_write(bsel, FMC176_AD9517, 0x0018, 0x07);
  fmc176_spi_write(bsel, FMC176_AD9517, 0x0232, 0x01);
  delay_ms(1);

  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0000, 0x18);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0001, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0003, 0x51);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0004, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0010, 0x7c);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0011, 0x01);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0012, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0013, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0014, 0x05);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0015, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0016, 0x05);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0017, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0018, 0x07);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0019, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x001a, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x001b, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x001c, 0x02);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x001d, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x001e, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a0, 0x01);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a1, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a2, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a3, 0x01);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a4, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a5, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a6, 0x01);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a7, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a8, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00a9, 0x01);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00aa, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00ab, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00f0, 0x08);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00f1, 0x08);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00f4, 0x08);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x00f5, 0x08);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0140, 0x42);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0141, 0x42);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0142, 0x42);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0143, 0x43);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0190, 0x21);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0191, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0192, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0196, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0197, 0x80);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0198, 0x02);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0199, 0x21);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019a, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019b, 0x11);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019c, 0x20);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019d, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019e, 0x21);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x019f, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x01a0, 0x11);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x01a1, 0x20);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x01a2, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x01e0, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x01e1, 0x02);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0230, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0231, 0x00);
  fmc176_spi_verify(bsel, FMC176_AD9517, 0x0232, 0x00);

  if ((fmc176_spi_read(bsel, FMC176_AD9517, 0x001f, 0x1) & 0x1) == 0x0) {
    xil_printf("AD9517, PLL errors!!\n\r");
    return(1);
  }

  return(0);
}

void adc_capture(u32 bsel, u32 sel, u32 qwcnt, u32 sa) {
  u32 dma_baseaddr;
  u32 cf_baseaddr;
  if (bsel == 1) {
    dma_baseaddr = (sel == 1) ? DMA1_1_BASEADDR : DMA1_0_BASEADDR;
    cf_baseaddr = (sel == 1) ? CF_ADC1_1_BASEADDR : CF_ADC1_0_BASEADDR;
  } else {
    dma_baseaddr = (sel == 1) ? DMA0_1_BASEADDR : DMA0_0_BASEADDR;
    cf_baseaddr = (sel == 1) ? CF_ADC0_1_BASEADDR : CF_ADC0_0_BASEADDR;
  }
  Xil_Out32((dma_baseaddr + 0x030), 0); // clear dma operations
  Xil_Out32((dma_baseaddr + 0x030), 1); // enable dma operations
  Xil_Out32((dma_baseaddr + 0x048), sa); // capture start address
  Xil_Out32((dma_baseaddr + 0x058), (qwcnt * 8)); // number of bytes
  Xil_Out32((cf_baseaddr + 0x00c), 0x0); // capture disable
  Xil_Out32((cf_baseaddr + 0x010), 0xf); // clear status
  Xil_Out32((cf_baseaddr + 0x014), 0xf); // clear status
  Xil_Out32((cf_baseaddr + 0x00c), (0x10000 | (qwcnt - 1)));
  do {delay_ms(1);}
  while ((Xil_In32(cf_baseaddr + 0x010) & 0x1) == 1);
  if ((Xil_In32(cf_baseaddr + 0x010) & 0x02) == 0x02) {
    xil_printf("overflow occured, capture may be corrupted\n\r");
  }
  microblaze_flush_dcache();
  microblaze_invalidate_dcache();
}

void adc_test(u32 bsel, u32 sel, u32 mode, u32 format) {
  u32 i;
  u32 n;
  u32 rdata_2;
  u32 rdata[2];
  u32 edata[2];
  u32 cf_baseaddr;
  edata[0] = 0;
  edata[1] = 0;
  fmc176_spi_write(bsel, ad9250_sel(sel), 0x14, format);
  fmc176_spi_write(bsel, ad9250_sel(sel), 0x0d, mode);
  fmc176_spi_write(bsel, ad9250_sel(sel), 0xff, 0x01);
  adc_capture(bsel, sel, 16, DDR_BASEADDR);
  xil_printf("adc_test: board(%d), sel(%d), mode(%2d), format(%2d)\n\r", bsel, sel, mode, format);
  if (bsel == 1) {
    cf_baseaddr = (sel == 1) ? CF_ADC1_1_BASEADDR : CF_ADC1_0_BASEADDR;
  } else {
    cf_baseaddr = (sel == 1) ? CF_ADC0_1_BASEADDR : CF_ADC0_0_BASEADDR;
  }
  if ((mode == 0x5) || (mode == 0x6)) {
    if (format == 0x1) return;
    Xil_Out32((cf_baseaddr+0x24), ((mode == 0x5) ? 0x03 : 0x00));
    delay_ms(10);
    Xil_Out32((cf_baseaddr+0x14), 0xffff);
    delay_ms(100);
    if ((Xil_In32(cf_baseaddr+0x14) & 0xffff) != 0) {
      xil_printf("  ERROR: PN status(%04x).\n\r", Xil_In32(cf_baseaddr+0x14));
    }
    return;
  }
  for (n = 0; n < 32; n++) {
    rdata_2 = Xil_In32(DDR_BASEADDR+(n*4));
    rdata[0] = (rdata_2>> 0) & 0xffff;
    rdata[1] = (rdata_2>>16) & 0xffff;
    for (i = 0; i < 2; i++) {
      if ((mode == 0x1) && (format == 0)) edata[i] = 0x2000;
      if ((mode == 0x2) && (format == 0)) edata[i] = 0x3fff;
      if ((mode == 0x3) && (format == 0)) edata[i] = 0x0000;
      if ((mode == 0x1) && (format == 1)) edata[i] = 0x0000;
      if ((mode == 0x2) && (format == 1)) edata[i] = 0x1fff;
      if ((mode == 0x3) && (format == 1)) edata[i] = 0x2000;
      if (((mode == 0x4) || (mode == 0x7)) && (n == 0)) edata[i] = (rdata[i] & 0xffff);
      if ((mode == 0x4) && (n > 0)) edata[i] = (edata[i] == 0x2aaa) ? 0x1555 : 0x2aaa;
      if ((mode == 0x7) && (n > 0)) edata[i] = (edata[i] == 0x3fff) ? 0x0000 : 0x3fff;
      if (rdata[i] != edata[i]) {
        xil_printf("  ERROR[%2d]: sel(%d), ch(%d), rcv(%08x), exp(%08x)\n\r", n, sel, i, rdata[i], edata[i]);
      }
    }
  }
}

u32 ad9250_setup(u32 bsel) {

  u32 sel;
  u32 mode;
  u32 jesd_baseaddr;

  jesd_baseaddr = (bsel == 1) ? JESD1_BASEADDR : JESD0_BASEADDR;

  Xil_Out32((jesd_baseaddr + 0x14), 0x30); // reset
  delay_ms(100);

  Xil_Out32((jesd_baseaddr + 0x04), 0x000c); // lanesync and scramble
  Xil_Out32((jesd_baseaddr + 0x0c), 0x1f01); // frm and byte counts

  for (sel = 0; sel < 2; sel++) {
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x18, 0x0f); // max vref
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x64, 0xf0); // did
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x80, 0x0f); // powerdown
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x5f, 0x17); // char repl disabled & ilas
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x09, 0x01); // clock control
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x82, 0x02); // lane b = 0, lane a = 2
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x83, 0x31); // lane c = 1, lane d = 3
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x6e, 0x81); // scr, 2-lane
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x70, 0x1f); // no. of frames per multi frame
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x5e, 0x22); // m=2, l=2
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x5f, 0x16); // char repl disabled & ilas
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x80, 0x09); // powerdown
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x3a, 0x10); // sysref ctrl
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x14, 0x00); // offset binary
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x0d, 0x00); // test patterns
    fmc176_spi_write(bsel, ad9250_sel(sel), 0xff, 0x01);
    fmc176_spi_write(bsel, ad9250_sel(sel), 0xff, 0x00);

    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x18, 0x0f); // max vref
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x64, 0xf0); // did
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x09, 0x01); // clock control
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x82, 0x02); // lane b = 0, lane a = 2
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x83, 0x31); // lane c = 1, lane d = 3
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x6e, 0x81); // scr, 2-lane
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x70, 0x1f); // no. of frames per multi frame
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x5f, 0x16); // char repl & ilas
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x80, 0x09); // powerdown
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x3a, 0x10); // sysref ctrl
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x14, 0x00); // offset binary
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x0d, 0x00); // test patterns
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x80, 0x09); // lane enable
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x5f, 0x16); // transmit enable
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x70, 0x1f); // frame count
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x6f, 0x01); // bytes per frame
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x6e, 0x81); // scrambler & lanes per link
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x61, 0x00); // test mode
    fmc176_spi_verify(bsel, ad9250_sel(sel), 0x60, 0x00); // sync mode
    if (fmc176_spi_read(bsel, ad9250_sel(sel), 0x0a, 0x1) != 0x81) {
      xil_printf("AD9250_%d, PLL/link errors!!\n\r", sel);
    }
  }

  for (sel = 0; sel < 2; sel++) {
    for (mode = 1; mode <= 7; mode++) {
      adc_test(bsel, sel, mode, 0x0);
      adc_test(bsel, sel, mode, 0x1);
    }
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x14, 0x00);
    fmc176_spi_write(bsel, ad9250_sel(sel), 0x0d, 0x00);
    fmc176_spi_write(bsel, ad9250_sel(sel), 0xff, 0x01);
  }

  return(0);
}

// ***************************************************************************
// ***************************************************************************

int main() {

  init_platform();
  lcd_display();

  fmc176_spi_read(0, FMC176_AD9517,    0x03, 0x1);
  fmc176_spi_read(0, FMC176_AD9250_0,  0x01, 0x1);
  fmc176_spi_read(0, FMC176_AD9250_1,  0x01, 0x1);
  fmc176_spi_read(1, FMC176_AD9517,    0x03, 0x1);
  fmc176_spi_read(1, FMC176_AD9250_0,  0x01, 0x1);
  fmc176_spi_read(1, FMC176_AD9250_1,  0x01, 0x1);

  ad9517_setup(0);
  ad9517_setup(1);
  ad9250_setup(0);
  ad9250_setup(1);

  xil_printf("done\n\r");
  cleanup_platform();
  return(0);
}


// ***************************************************************************
// ***************************************************************************

